@@ -1,4 +1,4 @@
-# Copyright (C) 2017  The Software Heritage developers
+# Copyright (C) 2017-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -6,11 +6,14 @@
 """Utilities helper functions"""
 
 import datetime
+import logging
 import os
 import shutil
 import tempfile
+from typing import Dict, Optional
 
 from swh.core import tarball
+from swh.model.model import SnapshotBranch
 
 
 def init_git_repo_from_archive(project_name, archive_path, root_temp_dir="/tmp"):
@@ -73,3 +76,46 @@ def check_date_time(timestamp):
     if not timestamp:
         return None
     datetime.datetime.fromtimestamp(timestamp, datetime.timezone.utc)
+
+
+def ignore_branch_name(branch_name: bytes) -> bool:
+    """Should the git loader ignore the branch named `branch_name`?"""
+    if branch_name.endswith(b"^{}"):
+        # Peeled refs make the git protocol explode
+        return True
+    elif branch_name.startswith(b"refs/pull/") and branch_name.endswith(b"/merge"):
+        # We filter-out auto-merged GitHub pull requests
+        return True
+
+    return False
+
+
+def filter_refs(refs: Dict[bytes, bytes]) -> Dict[bytes, bytes]:
+    """Filter the refs dictionary using the policy set in `ignore_branch_name`"""
+    return {
+        name: target for name, target in refs.items() if not ignore_branch_name(name)
+    }
+
+
+def warn_dangling_branches(
+    branches: Dict[bytes, Optional[SnapshotBranch]],
+    dangling_branches: Dict[bytes, bytes],
+    logger: logging.Logger,
+    origin_url: str,
+) -> None:
+    dangling_branches = {
+        target: ref for target, ref in dangling_branches.items() if not branches[target]
+    }
+
+    if dangling_branches:
+        descr = [f"{ref!r}->{target!r}" for target, ref in dangling_branches.items()]
+
+        logger.warning(
+            "Dangling symbolic references: %s",
+            ", ".join(descr),
+            extra={
+                "swh_type": "swh_loader_git_dangling_symrefs",
+                "swh_refs": descr,
+                "origin_url": origin_url,
+            },
+        )
